# TPK4186 - Advanced Tools for Performance Engineering
# Problem: Morse player

# Required modules
import sys
import re
import time
import winsound

# Morse code
morseCode = {
  "A": ".-",
  "B": "-...",
  "C": "-.-.",
  "D": "-..",
  "E": ".",
  "F": "..-.",
  "G": "--.",
  "H": "....",
  "I": "..",
  "J": ".--",
  "K": "-.-",
  "L": ".-..",
  "M": "--",
  "N": "-.",
  "O": "---",
  "P": ".--.",
  "Q": "--.-",
  "R": ".-.",
  "S": "...",
  "T": "-",
  "U": "..-",
  "V": "...-",
  "W": ".--",
  "X": "-..-",
  "Y": "-.--",
  "Z": "--..",
  "0": "-----",
  "1": ".----",
  "2": "..---",
  "3": "...--",
  "4": "....-",
  "5": ".....",
  "6": "-....",
  "7": "--...",
  "8": "---..",
  "9": "----.",
  ".": ".-.-.-",
  ",": "--.--",
  "?": "..--..",
  "'": ".----.",
  "!": "-.-.--",
  "/": "-..-.",
  "(": "-.--.",
  ")": "-.--.-",
  "&": ".-...",
  ":": "---...",
  ";": "-.-.-.",
  "=": "-...-",
  "+": ".-.-.",
  "-": "-....-",
  "_": "..--.-",
  "\"": ".-..-.",
  "$": "...-..-",
  "@": ".--.-.",
  "\n": " ",
  " ": " "
  }

# Morse Encoder
def MorseEncodeMessage(sourceMessage):
  global morseCode
  targetMessage = ""
  first = True
  for sourceCharacter in sourceMessage:
    if first:
      first = False
    else:
      targetMessage += " "
    targetCharacter = morseCode.get(sourceCharacter.upper(), None)
    if targetCharacter!=None:
      targetMessage += targetCharacter
  return targetMessage

# Morse Decoder
def MorseDecodeMessage(sourceMessage):
  global morseCode
  reverseCode = dict()
  for key in morseCode:
    value = morseCode[key]
    reverseCode[value] = key
  reverseCode["#"] = " "
  sourceMessage2 = re.sub("   ", " # ", sourceMessage)
  sourceCharacters = re.split(" ", sourceMessage2)
  targetMessage = ""
  for sourceCharacter in sourceCharacters:
    targetCharacter = reverseCode.get(sourceCharacter, None)
    if targetCharacter=="":
      targetMessage += " "
    elif targetCharacter!=None:
      targetMessage += targetCharacter
  return targetMessage
    
# Morse player
FREQUENCY = 2093
DOT_DURATION = 100

def MorsePlaySound(sound):
  if sound=="-":
    winsound.Beep(FREQUENCY, 3*DOT_DURATION)
  elif sound==".":
    winsound.Beep(FREQUENCY, DOT_DURATION)
  else:
    time.sleep(0.001*DOT_DURATION)

def MorsePlayMessage(message):
  for sound in message:
    MorsePlaySound(sound)
    time.sleep(0.001*DOT_DURATION)

# Main
sourceMessage1 = "SOS"
sourceMessage2 = "I love you!"
sourceMessage3 = "What hath God wrought?"
sourceMessage4 = """CQD THIS IS TITANIC.
WE HAVE STRUCK AN ICEBERG.
SINKING.
REQUEST IMMEDIATE ASSISTANCE.
WOMEN AND CHILDREN ONBOARD.
COME AT ONCE.
POSITION 41.44N 50.24W"""
sourceMessage = sourceMessage4
targetMessage = MorseEncodeMessage(sourceMessage)
print(targetMessage)
MorsePlayMessage(targetMessage)
retrievedMessage = MorseDecodeMessage(targetMessage)
print(retrievedMessage)

