# TPK4186 - Advanced Tools for Performance Engineering
# TrainRegularityChecker.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import sys
import re

def CheckFile(fileName):
  try:
    file = open(fileName, "r")
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return
  errorNumber = 0
  warningNumber = 0
  lineNumber = 0
  for line in file:
    lineNumber += 1
    if lineNumber==1:
      continue
    line = line.rstrip()
    tokens = line.split(",")
    if len(tokens)!=5:
      sys.stderr.write("Error at line " + str(lineNumber) + ": incorrect number of items\n")
      errorNumber += 1
      continue
    date = tokens[0]
    category = tokens[1]
    regularity = tokens[3]
    if not re.match(r"(19|20)\d\d[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])", date):
      sys.stderr.write("Error at line " + str(lineNumber) + ": date " + date + " is not in the right format (yyyy-mm-dd)\n")
      errorNumber += 1
    if not re.match(r"transilien|ter|intercites|tgv", category.lower()):
      sys.stderr.write("Error at line " + str(lineNumber) + ": unknown train category " + category + "\n")
      errorNumber += 1
    if regularity=="":
      sys.stderr.write("Warning at line " + str(lineNumber) + ": regularity is missing\n")
      warningNumber += 1
    elif not re.match(r"100(\.0*)?|[0-9]{1,2}(\.[0-9]*)?", regularity):
      sys.stderr.write("Error at line " + str(lineNumber) + ": incorrect regularity " + regularity + " (not a percentage)\n")
      errorNumber += 1
  sys.stdout.write("File " + fileName + " checked\n")
  sys.stdout.write(str(lineNumber) + " lines\n")
  sys.stdout.write(str(errorNumber) + " errors\n")
  sys.stdout.write(str(warningNumber) + " warnings\n")

#fileName = "regularite.csv"
fileName = "buggy.csv"
CheckFile(fileName)
